dofile ("gui/image.lua")

-- Global
local width, height = get_window_size()
local options = { backgroundclick = 0, name = 0, timer = 0, uke = 0, score = 0, hint = 0, feedback = 0 }
local items_per_shelf = math.floor((height-100)/80)

local BTN_UP = 1
local BTN_HOVER = 2
local BTN_DOWN = 3

local current_section = 1
local total_sections = 0
local current_shelf = 1
local total_shelves = 0

local confirm_none = 1

-- Data Initializing
local data_types = { "catid", "catname", "itemid", "itemname", "price", "qi", "tier", "rentable", "tradeable", "ingame", "colorid" }
local data_table = {}		-- this will store the item data obtained from torishop.txt
local data_table_lines = 0
local data_exists = false

function load_data()
	local file = io.open("torishop/torishop.txt") --'http://www.toribash.com/dump/store.php
	if (file == nil) then
		return
 	end

	for i, v in ipairs(data_types) do
		data_table[v] = {}
	end

	local current_line = 0
	for ln in file:lines() do
		if string.match(ln, "^PRODUCT") then
			segments = 12
			local data_stream = { ln:match(("([^\t]*)\t"):rep(segments)) }	-- split a tab separated string into an array
			current_line = current_line + 1
			for i, v in ipairs(data_types) do
				data_table[v][current_line] = data_stream[i + 1]
			end
		--[[ DEFUNCT --
		elseif string.match(ln, "^CREDIT") then
			ln = string.gsub(ln, "CREDIT\t", "")
			toriCredits = tonumber(ln) --]]
		end
	end
	file:close()

	if (current_line > 0) then
		data_table_lines = current_line
		data_exists = true
	end
end

local section_order = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 32, 28, 29, 30, 31}
local sections = {}	-- Refer to catid in Torishop.txt for section numbers used

function load_sections()
	-- Colors Section
	sections[1] = { name = "Force Colors", total_items = 0, try = true, selected_index = 0 }
	sections[2]  = { name = "Relax Colors", total_items = 0, try = true, selected_index = 0 }
	sections[3] = { name = "Primary Gradients", total_items = 0, try = true, selected_index = 0 }
	sections[4] = { name = "Secondary Gradients", total_items = 0, try = true, selected_index = 0 }
	sections[5]  = { name = "Torso Colors", total_items = 0, try = true, selected_index = 0 }
	sections[6]  = { name = "Blood Colors", total_items = 0, try = true, selected_index = 0 }
	sections[7] = { name = "Ghost Colors", total_items = 0, try = true, selected_index = 0 }
	sections[8] = { name = "Right Hand Motion Trails", total_items = 0, try = true, selected_index = 0 }
	sections[9] = { name = "Left Hand Motion Trails", total_items = 0, try = true, selected_index = 0 }
	sections[10] = { name = "Right Leg Motion Trails", total_items = 0, try = true, selected_index = 0 }
	sections[11] = { name = "Left Leg Motion Trails", total_items = 0, try = true, selected_index = 0 }
	sections[12] = { name = "DQ Rings", total_items = 0, try = true, selected_index = 0 }
	sections[13] = { name = "Grip Colors", total_items = 0, try = false, selected_index = 0 }
	sections[14] = { name = "Timers", total_items = 0, try = false, selected_index = 0 }
	sections[15] = { name = "User Text Colors", total_items = 0, try = false, selected_index = 0 }
	sections[16] = { name = "Emote Colors", total_items = 0, try = false, selected_index = 0 }
	sections[17] = { name = "Hair Styles", total_items = 0, try = true, selected_index = 0}
	sections[18] = { name = "Hair Colors", total_items = 0, try = true, selected_index = 0}
	-- Unlockables Section
	sections[19] = { name = "128x128 Textures", total_items = 0, try = false, selected_index = 0 }
	sections[20] = { name = "256x256 Textures", total_items = 0, try = false, selected_index = 0 }
	sections[21] = { name = "512x512 Textures", total_items = 0, try = false, selected_index = 0 }
	sections[22] = { name = "Misc Textures", total_items = 0, try = false, selected_index = 0 }
	sections[23] = { name = "Trail Textures", total_items = 0, try = false, selected_index = 0 }
	sections[24] = { name = "GUI Textures", total_items = 0, try = false, selected_index = 0 }
	-- Miscellaneous Section
	--sections[25] = { name = "Starter Packs", total_items = 0, try = false, selected_index = 0 }
	--sections[26] = { name = "Premium Packs", total_items = 0, try = false, selected_index = 0 }
	--sections[27] = { name = "Hardcore Packs", total_items = 0, try = false, selected_index = 0 }
	sections[28] = { name = "QI", total_items = 0, try = false, selected_index = 0 }
	sections[29] = { name = "Misc", total_items = 0, try = false, selected_index = 0 }
	sections[30] = { name = "Collectibles", total_items = 0, try = false, selected_index = 0 }
	sections[31] = { name = "Toricredits", total_items = 0, try = false, selected_index = 0 }
	sections[32] = { name = "Sounds", total_items = 0, try = false, selected_index = 0 }
	
	for i, v in ipairs(section_order) do
		total_sections = total_sections + 1
	end
end
function load_items()
	for line = 1, data_table_lines do
		local id = data_table.catid[line] + 0
		local num = sections[id].total_items + 1
		sections[id][num] = {}
		sections[id][num].name = data_table.itemname[line]
		sections[id][num].price = data_table.price[line]
		sections[id][num].qi = data_table.qi[line]
		sections[id][num].color = data_table.colorid[line]
		sections[id][num].id = data_table.itemid[line]
		sections[id][num].tier = data_table.tier[line]
		sections[id].total_items = num
		line = line + 1
	end
end

local button_click_radius = 9
local buttons = {}

function load_buttons()
	-- Arrow-type buttons
	buttons.arrows = {}
	buttons.arrows.prev_section = { x = 65, y = 60, angle = 270, state = BTN_UP }
	buttons.arrows.next_section = { x = 300, y = 60, angle = 90, state = BTN_UP }
	buttons.arrows.prev_shelf = { x = 35, y = height/2, angle = 270, state = BTN_UP }
	buttons.arrows.next_shelf = { x = 415, y = height/2, angle = 90, state = BTN_UP }
	
	-- Text-type buttons
	buttons.torishop = { x = 20, y = height - 20, w = 130, h = 20, state = BTN_UP }
	buttons.none = { x = 345, y = 50, w = 45, h = 20, state = BTN_UP }


	for i = 1, items_per_shelf do
		buttons[i] = {}
		buttons[i].try = { x = 347, y = 20 + i*80, w = 60, h = 20, state = BTN_UP }
		buttons[i].buy = { x = 347, y = 40 + i*80, w = 60, h = 20, state = BTN_UP }
		--buttons[i].market = { x = 333, y = 50 + i*80, w = 60, h = 20, state = BTN_UP 	end
		buttons.confirm = { x = width/2 - 105, y = height/2 - 15, w = 205, h = 20, state = BTN_UP }
		buttons.cancel = { x = width/2 - 40, y = height/2 + 15, w = 80, h = 20, state = BTN_UP }
	end
end


-- Debugging
function check_items(num)	-- For debugging purposes only
	print(sections[num].name .. " : " .. sections[num].total_items .. " items")
	tmp = 1
	while (tmp <= sections[num].total_items) do
		print(sections[num][tmp].name .. " " .. sections[num][tmp].price .. " " .. sections[num][tmp].qi .. " " .. sections[num][tmp].color)
		tmp = tmp + 1
	end
end


-- Player
local player = {}

function init_player()
	local name = get_master().master.nick
	if (name ~= nil) then
		load_player(0, name)	-- use player customs if logged in
	end
	
	local data1 = set_torishop(0)
	player.blood = data1.blood_color
	player.torso = data1.torso_color
	player.ghost = data1.ghost_color
	player.impact = data1.impact_color
	player.pgrad = data1.grad_pri_color
	player.sgrad = data1.grad_sec_color
	player.lhmt = { data1.lh_trail_r, data1.lh_trail_g, data1.lh_trail_b, data1.lh_trail_a }
	player.rhmt = { data1.rh_trail_r, data1.rh_trail_g, data1.rh_trail_b, data1.rh_trail_a }
	player.llmt = { data1.ll_trail_r, data1.ll_trail_g, data1.ll_trail_b, data1.ll_trail_a }
	player.rlmt = { data1.rl_trail_r, data1.rl_trail_g, data1.rl_trail_b, data1.rl_trail_a }
	
	local data2 = get_joint_color(0, 0)
	player.relax = data2.joint.relax
	player.force = data2.joint.force
end

function set_player_color(item)
	local section_index = section_order[current_section]
	local name = sections[section_index].name
	local color = sections[section_index][item].color
	local itemname = sections[section][item].name

	if (name == "Force Colors") then
		set_joint_force_color(0, color)
	elseif (name == "Relax Colors") then
		set_joint_relax_color(0, color)
	elseif (name == "Primary Gradients") then
		set_gradient_primary_color(0, color)
	elseif (name == "Secondary Gradients") then
		set_gradient_secondary_color(0, color)
	elseif (name == "Torso Colors") then
		set_torso_color(0, color)
	elseif (name == "Blood Colors") then
		set_blood_color(0, color)
	elseif (name == "Ghost Colors") then
		set_ghost_color(0, color)
	--elseif (name == "Grip Colors") then
	elseif (name == "Left Hand Motion Trails") then
		set_separate_trail_color(0, 0, color)
	elseif (name == "Right Hand Motion Trails") then
		set_separate_trail_color(0, 1, color)
	elseif (name == "Left Leg Motion Trails") then
		set_separate_trail_color(0, 2, color)
	elseif (name == "Right Leg Motion Trails") then
		set_separate_trail_color(0, 3, color)
	elseif (name == "DQ Rings") then
		set_ground_impact_color(0, color)
	--elseif (name == "Timers") then
	--elseif (name == "User Text Colour") then
	--elseif (name == "Emote Colors") then
	elseif (name == "Hair Styles") then
	if (itemname == "Candy Locks") then
			set_hair_settings(0, 0, 1, 8, 125, 300, 0, 4, 3, 15, 30, 1, 1, 1, 15, 0, 40, 100)
			set_hair_settings(0, 1, 1, 0, 235, 295, 0, 4, 3, 15, 30, 5, 1, 1, 15, 0, 14, 100)
			set_hair_settings(0, 2, 1, 1, 0, 45, 0, 9, 4, 32, 40, 1, 1, 24, 15, 0, 24, 100)
			set_hair_settings(0, 3, 1, 8, 206, 230, 152, 5, 10, 20, 0, 17, 1, 50, 14, 0, 0, 100)
			set_hair_settings(0, 4, 1, 8, 184, 219, 178, 6, 10, 20, 0, 20, 5, 1, 5, 0, 0, 100)
			set_hair_settings(0, 5, 1, 8, 157, 221, 155, 5, 10, 20, 0, 18, 1, 36, 18, 0, 0, 100)
			set_hair_settings(0, 6, 1, 8, 360, 44, 180, 10, 10, 32, 38, 4, 25, 31, 56, 0, 0, 1)
			set_hair_settings(0, 7, 1, 1, 0, 66, 0, 3, 10, 46, 45, 16, 30, 1, 56, 0, 0, 100)
			set_hair_settings(0, 8, 1, 1, 0, 48, 0, 3, 10, 46, 45, 1, 34, 18, 56, 0, 0, 100)
			set_hair_settings(0, 9, 1, 1, 0, 33, 0, 3, 10, 46, 45, 1, 38, 18, 56, 0, 0, 100)
			set_hair_settings(0, 10, 1, 1, 0, 39, 0, 3, 10, 46, 40, 6, 43, 24, 56, 0, 0, 100)
			set_hair_settings(0, 11, 1, 1, 0, 29, 0, 3, 10, 41, 1, 41, 23, 1, 56, 0, 0, 100)
			set_hair_settings(0, 12, 1, 8, 314, 136, 223, 4, 10, 20, 1, 1, 16, 34, 56, 0, 0, 100)
			set_hair_settings(0, 13, 1, 0, 276, 103, 165, 5, 10, 20, 1, 2, 21, 48, 56, 0, 0, 3)
			set_hair_settings(0, 14, 1, 0, 269, 90, 165, 5, 10, 20, 1, 2, 21, 42, 56, 0, 0, 3)
			set_hair_settings(0, 15, 1, 8, 275, 274, 21, 5, 10, 20, 0, 2, 24, 32, 56, 0, 0, 3)
	elseif (itemname == "Cat Ears") then
			set_hair_settings(0, 0, 1, 8, 55, 80, 350, 3, 4, 30, 0, 1, 30, 1, 50, 5, 0, 100)
			set_hair_settings(0, 1, 1, 8, 305, 80, 10, 3, 4, 30, 0, 1, 30, 1, 50, 5, 0, 100)
			set_hair_settings(0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Centurion") then
			set_hair_settings(0, 0, 1, 8, 0, 0, 360, 6, 10, 30, 38, 2, 4, 2, 0, 8, 0, 50)
			set_hair_settings(0, 1, 1, 8, 0, 30, 360, 6, 10, 30, 38, 4, 6, 1, 0, 8, 0, 50)
			set_hair_settings(0, 2, 1, 8, 0, 60, 360, 6, 10, 30, 38, 4, 6, 1, 0, 0, 0, 50)
			set_hair_settings(0, 3, 1, 8, 0, 90, 360, 6, 10, 30, 38, 4, 6, 1, 66, 0, 0, 50)
			set_hair_settings(0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Daimyo Hat and Scarf") then
			set_hair_settings(0, 0, 1, 1, 0, 95, 0, 3, 10, 80, 20, 1, 56, 1, 16, 10, 0, 0)
			set_hair_settings(0, 1, 1, 8, 0, 297, 0, 6, 3, 24, 46, 18, 1, 100, 16, 12, 48, 40)
			set_hair_settings(0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Gentleman's Essentials") then
			set_hair_settings(0, 0, 1, 1, 0, 65, 0, 3, 10, 60, 60, 20, 1, 1, 7, 11, 0, 1)
			set_hair_settings(0, 1, 1, 1, 0, 65, 0, 3, 10, 46, 46, 44, 56, 1, 7, 11, 0, 1)
			set_hair_settings(0, 2, 1, 1, 100, 155, 0, 3, 10, 25, 0, 44, 1, 1, 7, 11, 57, 34)
			set_hair_settings(0, 3, 1, 8, 100, 140, 0, 4, 7, 10, 5, 3, 3, 100, 7, 11, 0, 1)
			set_hair_settings(0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Jiyu Dreads") then
			set_hair_settings(0, 0, 1, 8, 2, 86, 0, 9, 10, 14, 10, 3, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 1, 1, 8, 25, 82, 0, 9, 10, 14, 10, 3, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 2, 1, 8, 335, 82, 0, 9, 10, 14, 10, 3, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 3, 1, 8, 50, 86, 0, 9, 10, 14, 10, 3, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 4, 1, 8, 310, 86, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 5, 1, 8, 0, 70, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 6, 1, 8, 15, 70, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 7, 1, 8, 345, 70, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 8, 1, 8, 25, 35, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 9, 1, 8, 335, 35, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 10, 1, 8, 52, 57, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 11, 1, 8, 308, 57, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 12, 1, 8, 0, 0, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 13, 1, 8, 65, 35, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 14, 1, 8, 295, 35, 0, 9, 10, 14, 10, 2, 26, 32, 52, 7, 0, 6)
			set_hair_settings(0, 15, 1, 8, 358, 86, 0, 9, 10, 14, 10, 3, 26, 32, 52, 7, 0, 6)
	elseif (itemname == "Minihawk") then
			set_hair_settings(0, 0, 1, 8, 0, 10, 0, 3, 5, 18, 22, 4, 30, 1, 50, 10, 0, 50)
			set_hair_settings(0, 1, 1, 8, 0, 20, 0, 3, 5, 18, 22, 4, 30, 1, 50, 10, 0, 50)
			set_hair_settings(0, 2, 1, 8, 0, 30, 0, 3, 5, 18, 22, 4, 30, 1, 50, 10, 0, 50)
			set_hair_settings(0, 3, 1, 8, 0, 40, 0, 3, 5, 18, 22, 4, 30, 1, 50, 10, 0, 50)
			set_hair_settings(0, 4, 1, 8, 0, 50, 0, 3, 5, 18, 22, 4, 30, 1, 50, 10, 0, 50)
			set_hair_settings(0, 5, 1, 8, 0, 60, 0, 3, 5, 18, 22, 4, 30, 1, 50, 10, 0, 50)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Mohawk") then
			set_hair_settings(0, 0, 1, 8, 0, 0, 0, 4, 5, 30, 32, 1, 1, 1, 50, 7, 54, 54)
			set_hair_settings(0, 1, 1, 8, 0, 20, 0, 4, 5, 30, 32, 1, 1, 1, 50, 10, 0, 50)
			set_hair_settings(0, 2, 1, 8, 0, 40, 0, 4, 5, 30, 32, 1, 1, 1, 50, 10, 0, 50)
			set_hair_settings(0, 3, 1, 8, 0, 60, 0, 4, 5, 30, 32, 1, 1, 1, 50, 10, 0, 50)
			set_hair_settings(0, 4, 1, 8, 0, 80, 0, 4, 5, 30, 32, 1, 1, 1, 50, 10, 0, 50)
			set_hair_settings(0, 5, 1, 8, 0, 100, 0, 4, 5, 30, 32, 1, 1, 1, 50, 10, 0, 50)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Pony Tail") then
			set_hair_settings(0, 0, 1, 8, 0, 60, 180, 8, 10, 15, 25, 10, 1, 10, 2, 1, 0, 50)
			set_hair_settings(0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Spiky Hair") then
			set_hair_settings(0, 0, 1, 8, 0, 0, 0, 5, 4, 20, 0, 1, 1, 20, 38, 1, 0, 50)
			set_hair_settings(0, 1, 1, 8, 0, 30, 0, 5, 4, 20, 0, 1, 1, 20, 38, 1, 0, 50)
			set_hair_settings(0, 2, 1, 8, 0, 60, 0, 5, 4, 20, 0, 1, 1, 20, 38, 1, 0, 50)
			set_hair_settings(0, 3, 1, 8, 0, 90, 0, 5, 4, 20, 0, 1, 1, 20, 38, 1, 0, 50)
			set_hair_settings(0, 4, 1, 8, 0, 120, 0, 5, 4, 20, 0, 1, 1, 20, 38, 1, 0, 50)
			set_hair_settings(0, 5, 1, 8, 0, 150, 0, 5, 4, 20, 0, 1, 1, 20, 38, 1, 0, 50)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Surfer's Seishin") then
			set_hair_settings(0, 0, 1, 8, 0, 100, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 1, 1, 8, 30, 80, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 2, 1, 8, 60, 90, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 3, 1, 8, 90, 70, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 4, 1, 8, 330, 80, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 5, 1, 8, 300, 90, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 6, 1, 8, 270, 70, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 7, 1, 8, 0, 50, 0, 4, 10, 31, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 8, 1, 8, 0, 0, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 9, 1, 8, 50, 45, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 10, 1, 8, 310, 45, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 11, 1, 8, 115, 30, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 12, 1, 8, 245, 30, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 13, 1, 8, 30, 110, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 14, 1, 8, 330, 110, 0, 4, 10, 30, 0, 1, 1, 1, 0, 0, 0, 76)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "The Sultan's Fez") then
			set_hair_settings(0, 0, 1, 1, 0, 70, 0, 10, 10, 30, 25, 40, 16, 10, 2, 0, 0, 1)
			set_hair_settings(0, 1, 1, 0, 0, 70, 180, 8, 10, 0, 19, 17, 5, 6, 18, 0, 0, 55)
			set_hair_settings(0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Santa Hat") then
			set_hair_settings(0, 0, 1, 2, 146, 295, 280, 4, 10, 43, 0, 10, 28, 42, 68, 0, 0, 54)
			set_hair_settings(0, 1, 1, 1, 145, 291, 0, 5, 10, 46, 44, 6, 21, 16, 0, 0, 0, 100)
			set_hair_settings(0, 2, 1, 5, 144, 296, 280, 4, 10, 0, 19, 11, 26, 50, 0, 0, 46, 54)
			set_hair_settings(0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Santa Beard") then
			set_hair_settings(0, 0, 1, 3, 340, 250, 315, 4, 10, 30, 0, 15, 30, 20, 0, 0, 0, 100)
			set_hair_settings(0, 1, 1, 3, 20, 250, 45, 4, 10, 30, 0, 15, 30, 20, 0, 0, 0, 100)
			set_hair_settings(0, 2, 1, 3, 0, 247, 0, 4, 10, 30, 0, 20, 30, 20, 0, 0, 0, 100)
			set_hair_settings(0, 3, 1, 1, 342, 210, 240, 10, 10, 17, 0, 45, 1, 10, 0, 0, 0, 100)
			set_hair_settings(0, 4, 1, 1, 18, 210, 120, 10, 10, 17, 0, 45, 1, 10, 0, 0, 0, 100)
			set_hair_settings(0, 5, 1, 1, 330, 220, 285, 10, 10, 19, 0, 45, 2, 10, 0, 0, 0, 100)
			set_hair_settings(0, 6, 1, 1, 30, 220, 75, 10, 10, 19, 0, 45, 2, 10, 0, 0, 0, 100)
			set_hair_settings(0, 7, 1, 1, 220, 130, 55, 10, 10, 20, 0, 45, 8, 10, 0, 0, 0, 100)
			set_hair_settings(0, 8, 1, 1, 140, 130, 305, 10, 10, 20, 0, 45, 8, 10, 0, 0, 0, 100)
			set_hair_settings(0, 9, 1, 1, 135, 115, 310, 10, 10, 20, 0, 45, 15, 10, 0, 0, 0, 100)
			set_hair_settings(0, 10, 1, 1, 225, 115, 50, 10, 10, 20, 0, 45, 15, 10, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Uke Hair") then
			set_hair_settings(0, 0, 1, 8, 0, 55, 0, 3, 10, 30, 0, 1, 1, 1, 28, 0, 0, 100)
			set_hair_settings(0, 1, 1, 0, 0, 55, 0, 3, 10, 10, 25, 9, 1, 1, 28, 0, 30, 100)
			set_hair_settings(0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	elseif (itemname == "Tori Hair") then
			set_hair_settings(0, 0, 1, 6, 0, 50, 0, 4, 8, 15, 20, 1, 1, 1, 2, 0, 0, 52)
			set_hair_settings(0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 3, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 4, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 5, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 7, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 11, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 14, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			set_hair_settings(0, 15, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
	end
		reset_hair(0)
	elseif (name == "Hair Colors") then
		set_hair_color(0, color)
	end
end
function set_player_default_color()
	local section_index = section_order[current_section]
	local name = sections[section_index].name
	
	if (name == "Force Colors") then
		set_joint_force_color(0, player.force)
	elseif (name == "Relax Colors") then
		set_joint_relax_color(0, player.relax)
	elseif (name == "Primary Gradients") then
		set_gradient_primary_color(0, player.pgrad)
	elseif (name == "Secondary Gradients") then
		set_gradient_secondary_color(0, player.sgrad)
	elseif (name == "Torso Colors") then
		set_torso_color(0, player.torso)
	elseif (name == "Blood Colors") then
		set_blood_color(0, player.blood)
	elseif (name == "Ghost Colors") then
		set_ghost_color(0, player.ghost)
	--elseif (name == "Grip Colors") then
	elseif (name == "Left Hand Motion Trails") then
		set_separate_trail_color_2(0, 0, player.lhmt[1], player.lhmt[2], player.lhmt[3], player.lhmt[4])
	elseif (name == "Right Hand Motion Trails") then
		set_separate_trail_color_2(0, 1, player.rhmt[1], player.rhmt[2], player.rhmt[3], player.rhmt[4])
	elseif (name == "Left Leg Motion Trails") then
		set_separate_trail_color_2(0, 2, player.llmt[1], player.llmt[2], player.llmt[3], player.llmt[4])
	elseif (name == "Right Leg Motion Trails") then
		set_separate_trail_color_2(0, 3, player.rlmt[1], player.rlmt[2], player.rlmt[3], player.rlmt[4])
	elseif (name == "DQ Rings") then
		set_ground_impact_color(0, player.impact)
	--elseif (name == "Timers") then
	--elseif (name == "User Text Colour") then
	--elseif (name == "Emote Colors") then
	--elseif (name == "Hair Styles") then
	--elseif (name == "Hair Colors") then
	end
end


-- Internal
function change_section(num)
	current_section = current_section + num
	if (current_section < 1) then
		current_section = total_sections
	elseif (current_section > total_sections) then
		current_section = 1
	end

	section = section_order[current_section]
	items = sections[section].total_items
	total_shelves = math.ceil(items/items_per_shelf)
	current_shelf = 1
	if (sections[section].selected_index > 0) then
		current_shelf = math.floor(sections[section].selected_index/items_per_shelf) + 1
	end	
end
function change_shelf(num)
	current_shelf = current_shelf + num
	if (current_shelf < 1) then
		current_shelf = total_shelves
	elseif (current_shelf > total_shelves) then
		current_shelf = 1
	end
end

function select_color(r1, g1, b1, a1, marked, r2, g2, b2, a2)
	if (marked == true) then
		set_color(r2, g2, b2, a2)
	else
		set_color(r1, g1, b1, a1)
	end
end
function select_color_links(state)
	if (state == BTN_UP) then
		set_color(0.0, 0.0, 0.0, 1.0)
	elseif (state == BTN_DOWN) then
		set_color(1.0, 0.0, 0.0, 1.0)
	else
		set_color(0.0, 0.0, 1.0, 1.0)
	end
end

local MOUSE_UP = 0
local MOUSE_DOWN = 1
local mouse_state = MOUSE_UP

function mouse_down(mouse_btn, x, y)
	mouse_state = MOUSE_DOWN
	
	local r = button_click_radius
	for i, v in pairs(buttons.arrows) do
		if (x > (v.x - r) and x < (v.x + r) and y > (v.y - r) and y < (v.y + r)) then
			v.state = BTN_DOWN
		end
	end

	if (x > buttons.torishop.x and x < (buttons.torishop.x + buttons.torishop.w) and y > buttons.torishop.y and y < (buttons.torishop.y + buttons.torishop.h)) then
		buttons.torishop.state = BTN_DOWN
	end
	if (x > buttons.none.x and x < (buttons.none.x + buttons.none.w) and y > buttons.none.y and y < (buttons.none.y + buttons.none.h)) then
		buttons.none.state = BTN_DOWN
	end
	
	for i = 1, items_per_shelf do
		if (x > buttons[i].try.x and x < (buttons[i].try.x + buttons[i].try.w) and y > buttons[i].try.y and y < (buttons[i].try.y + buttons[i].try.h)) then
			buttons[i].try.state = BTN_DOWN
		end
		if (x > buttons[i].buy.x and x < (buttons[i].buy.x + buttons[i].buy.w) and y > buttons[i].buy.y and y < (buttons[i].buy.y + buttons[i].buy.h)) then
			buttons[i].buy.state = BTN_DOWN
		end
		--if (x > buttons[i].market.x and x < (buttons[i].market.x + buttons[i].market.w) and y > buttons[i].market.y and y < (buttons[i].market.y + buttons[i].market.h)) then
		--	buttons[i].market.state = BTN_DOWN
		--end
	end
	
	if confirm_none == 0 then
		if (x > buttons.confirm.x and x < (buttons.confirm.x + buttons.confirm.w) and y > buttons.confirm.y and y < (buttons.confirm.y + buttons.confirm.h)) then
			buttons.confirm.state = BTN_DOWN
		end
		if (x > buttons.cancel.x and x < (buttons.cancel.x + buttons.cancel.w) and y > buttons.cancel.y and y < (buttons.cancel.y + buttons.cancel.h)) then
			buttons.cancel.state = BTN_DOWN
		end	
	end
end
function mouse_up(mouse_btn, x, y)
	mouse_state = MOUSE_UP
	local section_index = section_order[current_section]
	
	local r = button_click_radius
	for i, v in pairs(buttons.arrows) do
		if (x > (v.x - r) and x < (v.x + r) and y > (v.y - r) and y < (v.y + r)) then
			v.state = BTN_HOVER
			if (v == buttons.arrows.prev_section) then
				change_section(-1)
			elseif (v == buttons.arrows.next_section) then
				change_section(1)
			elseif (v == buttons.arrows.prev_shelf) then
				change_shelf(-1)
			elseif (v == buttons.arrows.next_shelf) then
				change_shelf(1)
			end
		end
	end

	if (x > buttons.torishop.x and x < (buttons.torishop.x + buttons.torishop.w) and y > buttons.torishop.y and y < (buttons.torishop.y + buttons.torishop.h)) then
		buttons.torishop.state = BTN_HOVER
		open_url("http://www.toribash.com/forum/tori_shop.php")
	end
	if (x > buttons.none.x and x < (buttons.none.x + buttons.none.w) and y > buttons.none.y and y < (buttons.none.y + buttons.none.h)) then
		buttons.none.state = BTN_HOVER
		if (sections[section_index].try == true) then
			sections[section_index].selected_index = 0
			set_player_default_color()
			
			if (sections[section_index].name == "Hair Styles") then
				set_hair_settings(0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
			end
			
			if (sections[section_index].name == "Hair Colors") then
				set_hair_color(0, 0)
			end
		end
	end
	for i = 1, items_per_shelf do
		local item_index = i + (current_shelf - 1)*items_per_shelf
		if (x > buttons[i].try.x and x < (buttons[i].try.x + buttons[i].try.w) and y > buttons[i].try.y and y < (buttons[i].try.y + buttons[i].try.h)) then
			buttons[i].try.state = BTN_HOVER
			if (sections[section_index].try == true) then
				sections[section_index].selected_index = item_index
				set_player_color(item_index)
			end
		end
		if (x > buttons[i].buy.x and x < (buttons[i].buy.x + buttons[i].buy.w) and y > buttons[i].buy.y and y < (buttons[i].buy.y + buttons[i].buy.h)) then
			buttons[i].buy.state = BTN_HOVER
			confirm_name = sections[section_index][item_index].name
			confirm_id = sections[section_index][item_index].id
			confirm_none = 0
		end
		--if (x > buttons[i].market.x and x < (buttons[i].market.x + buttons[i].market.w) and y > buttons[i].market.y and y < (buttons[i].market.y + buttons[i].market.h)) then
		--	buttons[i].market.state = BTN_HOVER
		--	open_url("http://forum.toribash.com/tori_market.php?action=search&item=" .. sections[section_index][item_index].name)
		--end
	end
	
	if confirm_none == 0 then
		if (x > buttons.confirm.x and x < (buttons.confirm.x + buttons.confirm.w) and y > buttons.confirm.y and y < (buttons.confirm.y + buttons.confirm.h)) then
			buttons.confirm.state = BTN_HOVER
			open_url("http://forum.toribash.com/" .. confirm_id)
			confirm_name = 0
			confirm_id = 0
			confirm_none = 1
		end
	
		if (x > buttons.cancel.x and x < (buttons.cancel.x + buttons.cancel.w) and y > buttons.cancel.y and y < (buttons.cancel.y + buttons.cancel.h)) then
			buttons.cancel.state = BTN_HOVER
			confirm_name = 0
			confirm_id = 0
			confirm_none = 1
		end	
	end
end
function mouse_move(x, y)
	local r = button_click_radius
	for i, v in pairs(buttons.arrows) do
		if (x > (v.x - r) and x < (v.x + r) and y > (v.y - r) and y < (v.y + r)) then
			if (mouse_state == MOUSE_DOWN) then
				v.state = BTN_DOWN
			else
				v.state = BTN_HOVER
			end
		else
			v.state = BTN_UP
		end
	end

	if (x > buttons.torishop.x and x < (buttons.torishop.x + buttons.torishop.w) and y > buttons.torishop.y and y < (buttons.torishop.y + buttons.torishop.h)) then
		if (mouse_state == MOUSE_DOWN) then
			buttons.torishop.state = BTN_DOWN
		else
			buttons.torishop.state = BTN_HOVER
		end
	else
		buttons.torishop.state = BTN_UP
	end
	if (x > buttons.none.x and x < (buttons.none.x + buttons.none.w) and y > buttons.none.y and y < (buttons.none.y + buttons.none.h)) then
		if (mouse_state == MOUSE_DOWN) then
			buttons.none.state = BTN_DOWN
		else
			buttons.none.state = BTN_HOVER
		end
	else
		buttons.none.state = BTN_UP
	end
		
	for i = 1, items_per_shelf do
		if (x > buttons[i].try.x and x < (buttons[i].try.x + buttons[i].try.w) and y > buttons[i].try.y and y < (buttons[i].try.y + buttons[i].try.h)) then
			if (mouse_state == MOUSE_DOWN) then
				buttons[i].try.state = BTN_DOWN
			else
				buttons[i].try.state = BTN_HOVER
			end
		else
			buttons[i].try.state = BTN_UP
		end
		if (x > buttons[i].buy.x and x < (buttons[i].buy.x + buttons[i].buy.w) and y > buttons[i].buy.y and y < (buttons[i].buy.y + buttons[i].buy.h)) then
			if (mouse_state == MOUSE_DOWN) then
				buttons[i].buy.state = BTN_DOWN
			else
				buttons[i].buy.state = BTN_HOVER
			end
		else
			buttons[i].buy.state = BTN_UP
		end
		--if (x > buttons[i].market.x and x < (buttons[i].market.x + buttons[i].market.w) and y > buttons[i].market.y and y < (buttons[i].market.y + buttons[i].market.h)) then
		--	if (mouse_state == MOUSE_DOWN) then
		--		buttons[i].market.state = BTN_DOWN
		--	else
		--		buttons[i].market.state = BTN_HOVER
		--	end
		--else
		--	buttons[i].market.state = BTN_UP
		--end
	end
	
	if confirm_none == 0 then
		if (x > buttons.confirm.x and x < (buttons.confirm.x + buttons.confirm.w) and y > buttons.confirm.y and y < (buttons.confirm.y + buttons.confirm.h)) then
			if (mouse_state == MOUSE_DOWN) then
				buttons.confirm.state = BTN_DOWN
			else
				buttons.confirm.state = BTN_HOVER
			end
		else
			buttons.confirm.state = BTN_UP
		end
	
		if (x > buttons.cancel.x and x < (buttons.cancel.x + buttons.cancel.w) and y > buttons.cancel.y and y < (buttons.cancel.y + buttons.cancel.h)) then
			if (mouse_state == MOUSE_DOWN) then
				buttons.cancel.state = BTN_DOWN
			else
				buttons.cancel.state = BTN_HOVER
			end
		else
			buttons.cancel.state = BTN_UP
		end
	end
end

function key_down(key)
	if (key == string.byte(' ')) then
		draw_ground_impact(0)		
		return 1
	end
end
function key_up(key)
	if (key ~= string.byte('z') and key ~= string.byte('x') and key ~= string.byte('c')) then
		return 1
	end
end


-- Drawing
function draw_item(section, item, pos, marked)	-- the section the item is from, the item index in the section, the shelf position, whether it is selected
	select_color(1, 1, 1, 0.4, marked, 0.6, 0.6, 0.6, 0.7)
	draw_quad(50, (pos*80) + 4, 350, 70)
	
	local name = sections[section][item].name
	local price = sections[section][item].price
	local qi = sections[section][item].qi
	local tier = sections[section][item].tier
		
	if (string.find(name, "Secondary Gradient")) then
		name = string.gsub(name, "Secondary Gradient", "Sec. Grad.")
	end
	if (string.find(name, "Primary Gradient")) then
		name = string.gsub(name, "Primary Gradient", "Pr. Grad.")
	end
	if (string.find(name, "RHMT")) then
		name = string.gsub(name, "RHMT", "Right Hand MT")
	end
	if (string.find(name, "LHMT")) then
		name = string.gsub(name, "LHMT", "Left Hand MT")
	end
	if (string.find(name, "RLMT")) then
		name = string.gsub(name, "RLMT", "Right Leg MT")
	end
	if (string.find(name, "LLMT")) then
		name = string.gsub(name, "LLMT", "Left Leg MT")
	end

	local spacing = 18
	
	select_color(0.0, 0.0, 0.0, 1.0, marked, 1.0, 1.0, 1.0, 1.0)
	if (tier ~= "None") then
	draw_text(name, 60, (pos*80) + 10)
	draw_text(name, 61, (pos*80) + 10)
	draw_text("Price: ", 60, (pos*80) + spacing + 15)
	draw_text(price, 105, (pos*80) + spacing + 15)
	draw_text("QI: ", 175, (pos*80) + (spacing*2) + 13)
	draw_text(qi, 199, (pos*80) + (spacing*2) + 13)
	draw_text("Tier: ", 60, (pos*80) + (spacing*2) + 13)
	draw_text(tier, 95, (pos*80) + (spacing*2) + 13)
	elseif ((tier == "None") and (qi ~= "0")) then
	draw_text(name, 60, (pos*80) + 10)
	draw_text(name, 61, (pos*80) + 10)
	draw_text("Price: ", 60, (pos*80) + spacing + 15)
	draw_text(price, 105, (pos*80) + spacing + 15)
	draw_text("QI: ", 60, (pos*80) + (spacing*2) + 13)
	draw_text(qi, 84, (pos*80) + (spacing*2) + 13)
	else	
	draw_text(name, 60, (pos*80) + 15)
	draw_text(name, 61, (pos*80) + 15)
	draw_text("Price: ", 60, (pos*80) + (spacing*2) + 10)
	draw_text(price, 105, (pos*80) + (spacing*2) + 10)
	end
	
	-- Item Icons
	set_color(0,0,0,1)
	draw_quad(265, (pos*80) + (spacing*2) - 30, 66, 66)
	set_color(1,1,1,1)
	icon = load_texture("/torishop/icons/"..name..".tga")
	draw_quad(266, (pos*80) + (spacing*2) - 29, 64, 64, icon)
	unload_texture(icon)
end

function draw_shelf()
	local section_index = section_order[current_section]

	-- Draw items on the shelf
	for i = 1, items_per_shelf do
		local item_index = i + (current_shelf - 1)*items_per_shelf
		if (item_index > sections[section_index].total_items) then
			break
		end
		local selected = false
		if (item_index == sections[section_index].selected_index) then
			selected = true
		end
		draw_item(section_index, item_index, i, selected)
		
		if (sections[section_index].try == true) then
			select_color_links(buttons[i].try.state)
			draw_text("TRY", buttons[i].try.x + 1, buttons[i].try.y)
			select_color_links(buttons[i].buy.state)
			draw_text("BUY", buttons[i].buy.x, buttons[i].buy.y)
			draw_text("BUY", buttons[i].buy.x + 1, buttons[i].buy.y)
		else
			select_color_links(buttons[i].buy.state)
			draw_text("BUY", buttons[i].buy.x, buttons[i].buy.y - 5)
			draw_text("BUY", buttons[i].buy.x + 1, buttons[i].buy.y - 5)
		end
		--select_color_links(buttons[i].market.state)
		--draw_text("MARKET", buttons[i].market.x, buttons[i].market.y)
		
	end
	
	-- Draw shelf text
	if (sections[section_index].try == true) then
		select_color_links(buttons.none.state)
		draw_text("None", buttons.none.x, buttons.none.y)
	end
	
	set_color(0.0, 0.0, 0.0, 1.0)
	draw_text(sections[section_index].name, buttons.arrows.prev_section.x + 117 - string.len(sections[section_index].name)*3.3, buttons.arrows.prev_section.y - button_click_radius)
	draw_text(sections[section_index].name, buttons.arrows.prev_section.x + 118 - string.len(sections[section_index].name)*3.3, buttons.arrows.prev_section.y - button_click_radius)	
	draw_text("Page:  " .. current_shelf .. " of " .. total_shelves, 335, buttons.torishop.y) --same pos as the "visit the torishop"
end
function draw_torishop()
	-- Flat BG to hide echo
	set_color(1, 1, 1, 1)
	draw_quad(0, 0, 450, height)

	-- The shop borders	
	set_color(0.7, 0.7, 0.7, 0.6)
	draw_quad(0, 0, 20, height) -- Left
	draw_quad(430, 0, 20, height) -- Right
	draw_quad(20, 0, 410, 20) -- Top
	draw_quad(20, height - 20, 410, 20) -- Bottom

	-- The shop background
	set_color(0.2, 0.2, 1.0, 0.5)
	draw_quad(20, 20, 410, height-40) --Item list bg

	-- The shop title
	set_color(0, 0, 0, 1.0)
	draw_text("TORISHOP", width-270, 12, FONTS.BIG)
	set_color(0.6, 0.0, 0.0, 1.0)
	draw_text("TORISHOP", width-270, 13, FONTS.BIG)
	set_color(0.3, 0, 0, 1.0)
	draw_text("Every credit counts", width-245, 65, FONTS.MEDIUM)

	-- The Torishop link
	select_color_links(buttons.torishop.state)
	draw_text("VISIT THE TORISHOP", buttons.torishop.x + 4, buttons.torishop.y)
	draw_text("VISIT THE TORISHOP", buttons.torishop.x + 5, buttons.torishop.y)
	
	-- The current shelf
	draw_shelf()
		
	-- draw arrow buttons
	set_color(0.0, 0.0, 0.0, 1.0)
	for i, v in pairs(buttons.arrows) do
		if (v.state == BTN_HOVER) then
			draw_disk(v.x, v.y, 0, 13, 3, 1, v.angle, 360, 0)
		elseif (v.state == BTN_DOWN) then
			draw_disk(v.x, v.y, 0, 7, 3, 1, v.angle, 360, 0)
		else
			draw_disk(v.x, v.y, 0, 10, 3, 1, v.angle, 360, 0)
		end
	end

	-- Purchase confirmation
	if confirm_none == 0 then
		-- Draw confirmation window
		set_color(1,1,1,.9)
		draw_quad(width/2 - 250, height/2 - 90, 500, 150)
		set_color(0,0,0,1)
		draw_quad(width/2 - 250, height/2 - 90, 1, 150)
		draw_quad(width/2 + 250, height/2 - 90, 1, 150)
		draw_quad(width/2 - 250, height/2 - 90, 500, 1)
		draw_quad(width/2 - 250, height/2 + 60, 500, 1)
		
		set_color(0,0,0,1)
		
			if string.len(confirm_name) > 15 then
				draw_centered_text("Are you sure want to buy", buttons.confirm.y - 63, FONTS.MEDIUM)
				draw_centered_text(confirm_name .. "?", buttons.confirm.y - 40, FONTS.MEDIUM)
			else
				draw_centered_text("Are you sure want to buy " .. confirm_name .. "?", buttons.confirm.y - 50, FONTS.MEDIUM)
			end
			
		select_color_links(buttons.confirm.state)
		draw_centered_text("CONFIRM PURCHASE", buttons.confirm.y, FONTS.MEDIUM)
		select_color_links(buttons.cancel.state)
		draw_centered_text("CANCEL", buttons.cancel.y, FONTS.MEDIUM)
	end
end
function draw_offline()
end


-- Main
function close_torishop()
	print("closing torishop...")
	uninit_torishop()
	reset_camera(1)
	
	for i, v in pairs(options) do
		set_option(i, v) 
	end
	--run_cmd("ls rus.lua")
	run_cmd("opt chat 1")
	echo(" ")
	echo(" ")
	echo(" ")
	echo(" ")
	echo(" ")
	echo(" ")
	echo(" ")
	echo(" ")
    remove_hooks("torishop")
end
function init_torishop()
	start_torishop_camera(15)

	for i, v in pairs(options) do
		options[i] = get_option(i)
		set_option(i, 0)
	end
	
	-- Prepare the player figure
	init_player()
	
	-- Ensure Torishop can close properly
	add_hook("leave_game", "torishop", close_torishop)

	-- Prepare the item display
	load_data()
	if (data_exists == false) then
		print("No torishop")	-- print no torishop message
		
		add_hook("draw2d", "torishop", draw_offline)
		return
	end

	load_sections()
	load_items()
	load_buttons()
	
	change_section(0)
	change_shelf(0)
	
	
	add_hook("draw2d", "torishop", draw_torishop)
	add_hook("mouse_button_down", "torishop", mouse_down)
	add_hook("mouse_button_up", "torishop", mouse_up)
	add_hook("mouse_move", "torishop", mouse_move)
	add_hook("key_down", "torishop", key_down)
	add_hook("key_up", "torishop", key_up)
	run_cmd ("clear")
end


-- Run Torishop
init_torishop()

